@@ -1174,6 +1174,8 @@ DAHDI_IRQ_HANDLER(t1xxp_interrupt)
 	if (!ints)
 		return IRQ_NONE;
 
+	local_irq_save(flags);
+
 	outb(ints, wc->ioaddr + WC_INTSTAT);
 
 	if (!wc->intcount) {
@@ -1188,13 +1190,13 @@ DAHDI_IRQ_HANDLER(t1xxp_interrupt)
 		t1xxp_receiveprep(wc, ints);
 		t1xxp_transmitprep(wc, ints);
 	}
-	spin_lock_irqsave(&wc->lock, flags);
+	spin_lock(&wc->lock);
 
 #if 1
 	__handle_leds(wc);
 #endif
 
-	spin_unlock_irqrestore(&wc->lock, flags);
+	spin_unlock(&wc->lock);
 
 	/* Count down timers */
 	t1xxp_do_counters(wc);
@@ -1220,6 +1222,8 @@ DAHDI_IRQ_HANDLER(t1xxp_interrupt)
 	if (ints & 0x20)
 		printk(KERN_INFO "PCI Target abort\n");
 
+	local_irq_restore(flags);
+
 	return IRQ_RETVAL(1);
 }
 
@@ -1333,7 +1337,7 @@ static int __devinit t1xxp_init_one(struct pci_dev *pdev, const struct pci_devic
 	/* Keep track of which device we are */
 	pci_set_drvdata(pdev, wc);
 	
-	if (request_irq(pdev->irq, t1xxp_interrupt, DAHDI_IRQ_SHARED_DISABLED, "t1xxp", wc)) {
+	if (request_irq(pdev->irq, t1xxp_interrupt, IRQF_SHARED, "t1xxp", wc)) {
 		printk(KERN_NOTICE "t1xxp: Unable to request IRQ %d\n", pdev->irq);
 		kfree(wc);
 		return -EIO;
