@@ -25,17 +25,15 @@
 #include "xpd.h"
 
 enum fxs_opcodes {
-	XPROTO_NAME(FXS, SIG_CHANGED)		= 0x06,
-/**/
-	XPROTO_NAME(FXS, CHAN_POWER)		= 0x0F,	/* Write to SLIC */
-	XPROTO_NAME(FXS, CHAN_CID)		= 0x0F,	/* Write to SLIC */
-	XPROTO_NAME(FXS, LED)			= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, SIG_CHANGED) = 0x06,
+	 /**/ XPROTO_NAME(FXS, CHAN_POWER) = 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, CHAN_CID) = 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, LED) = 0x0F,	/* Write to SLIC */
 };
 
-
 DEF_RPACKET_DATA(FXS, SIG_CHANGED,
-	xpp_line_t	sig_status;	/* channels: lsb=1, msb=8 */
-	xpp_line_t	sig_toggles;	/* channels: lsb=1, msb=8 */
-	);
+		xpp_line_t sig_status;	/* channels: lsb=1, msb=8 */
+		xpp_line_t sig_toggles;	/* channels: lsb=1, msb=8 */
+		);
 
-#endif	/* CARD_FXS_H */
+#endif /* CARD_FXS_H */
